/*
 * This file is part of firk's window manager for x11 (fwm)
 * Copyright (C) 2016-2021 firk <firk@cantconnect.ru>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <unistd.h>
#include <sys/wait.h>
#include "main.h"
#include "base.h"
#include "config.h"
#include "winlist.h"
#include "taskbar.h"
#include "error.h"

time_t now;

extern void logprint(char const * fmt, ...) {
  char buf[110];
  va_list arg;
  va_start(arg, fmt); vsnprintf(buf, sizeof(buf), fmt, arg); va_end(arg);
  fprintf(stderr, "fwm: %s\n", buf);
}

extern void failprint(char const * fmt, ...) {
  char buf[110];
  va_list arg;
  va_start(arg, fmt); vsnprintf(buf, sizeof(buf), fmt, arg); va_end(arg);
  fprintf(stderr, "fwm: (fatal) %s\n", buf);
  exit(-1);
}

int main(int argc, char const * const * argv) {
  int status;
  XEvent ev;
  load_config((argc>=2)?argv[1]:NULL);
  base_init(NULL);
  taskbar_update();
  error_init();
  
  while(1) {
    XSync(display, False);
    time(&now);
    waitpid(-1, &status, WNOHANG);
    taskbar_periodic();
    if(!base_get_event(&ev)) { usleep(10000); continue; }
    if(taskbar_created) {
      if(ev.type==Expose && ev.xexpose.window==taskbar_window ||
         (ev.type==ButtonPress || ev.type==ButtonRelease) && ev.xbutton.window==taskbar_window ||
         ev.type==MotionNotify && ev.xmotion.window==taskbar_window ||
         ev.type==LeaveNotify && ev.xcrossing.window==taskbar_window) {
        taskbar_handle_event(&ev);
        continue;
      }
    }
    if(base_handle_event(&ev)) continue;
    logprint("got unhandled event %d", ev.type);
  }
}
